#!/bin/bash
#
# License: Copyright 2011 SpinetiX S.A. This file is licensed
#          under the terms of the GNU General Public License version 2.
#          This program is licensed "as is" without any warranty of any
#          kind, whether express or implied.
#
### BEGIN INIT INFO
# Required-Start:
# Required-Stop:
# Should-Start:
# Should-Stop:
# Default-Start: 2 3 4 5
# Default-Stop:
# Short-Description: Does pending media-mount actions
# Description: Does pending media-mount actions
### END INIT INFO
# chkconfig: 2345 13 0

# Init script information
INIT_NAME=media-mount.sh
DESC=""

# The state directory, needs to be under /dev because
# udev may start queuing events even before the root fs
# is read-write and /dev is the only guaranteed read-write place
STATEDIR=/dev/.media-mount

# The file with all the pending actions
PENDINGFILE="$STATEDIR"/pending-actions

# The file to indicate that everything is read and media-mount
# can do its work directly instead of entering pending actions
READYFILE="$STATEDIR"/ready

# The file to lock the state dir
STATELOCKFILE="$STATEDIR"/lock

# The maximum number of tries for locking the state dir
MAX_LOCK_TRIES=15

# The executable that handles the device events
DEVEXE=/etc/dev.d/block/media-mount.dev

state_lock() {
	for (( i=1; i <= $MAX_LOCK_TRIES; i++ )) ; do
		dotlockfile -p -r 0 -l "$STATELOCKFILE" && return 0
		sleep 1
	done
	echo "ERROR: failed acquiring state lock"
	return 1
}

state_unlock() {
	dotlockfile -p -u "$STATELOCKFILE"
}

do_pending_actions() {
    local oldifs="$IFS"
    local action
    local devname
    local physdevpath

    [ -f "$PENDINGFILE" ] || return 0

    while [ -f "$PENDINGFILE" ]; do
	if ! mv "$PENDINGFILE" "$PENDINGFILE".act; then
	    echo "ERROR: failed to move pending actions file" >&2
	    break
	fi
	# Process the previously pending actions without the lock
	# as the device handler invocation below may take long and
	# having the lock would prevent adding new device actions
	state_unlock
	IFS="${IFS}="
	while read key value; do
	    case "$key" in
		--START--)
		    action=
		    devname=
		    physdevpath=
		    ;;
		--END--)
		    logger -p daemon.info \
			-t "media-mount.sh [$$]" \
			"Processing pending '$action' '$devname' at '$physdevpath'"
		    SUBSYSTEM="block" \
			ACTION="$action" \
			DEVNAME="$devname" \
			PHYSDEVPATH="$physdevpath" \
			MEDIAMOUNTREADY="yes" \
			"$DEVEXE"
		    ;;
		ACTION)
		    action="$value"
		    ;;
		DEVNAME)
		    devname="$value"
		    ;;
		PHYSDEVPATH)
		    physdevpath="$value"
		    ;;
		*)
		    echo "WARNING: unrecognized key '$key' in pending actions" >&2
		    ;;
	    esac
	done < "$PENDINGFILE".act
	rm -f "$PENDINGFILE".act
	# Re-obtain the lock to examine if any new actions have been
	# queued as pending
	state_lock || break
    done
    IFS="$oldifs"
}

do_start() {

    # Make sure state dir exists
    [ -d "$STATEDIR" ] || mkdir -p "$STATEDIR"

    # Process all pending actions with state lock held
    if ! state_lock ; then
	# Cannot process stuff but at least let new actions run directly
	: > "$READYFILE"
	return
    fi

    do_pending_actions

    # From now on everything is setup for media-mount to do its own work
    : > "$READYFILE"

    state_unlock
}

#
# Main
#

case "$1" in
    start)
	echo -n "Processing pending media-mount actions"

	do_start

	echo "."

	;;
    stop)
	;;
esac
